/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2015 ESI-OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.


Application
    SbmFoam

Description
    Transient solver for incompressible, suspensions flow of spherical particles immersed in a Newtonian-Fluid
    (note: particles considered as a continuum medium but of concentration "c" ).
see: 
    T. DBOUK, L. LOBRY, E. LEMAIRE, and F. MOUKALLED, ‘Shear-induced Particles Migration.
    ‘Predictions from Experimental Determination of The Particle Stress Tensor’, 
    Journal of Non-Newtonian fluid Mechanics, 198, pp. 78-95, August 2013.


Application Written by:

Talib DBOUK
email@talibdbouk.com
www.talibdbouk.com

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"
#   include "createFields.H"
#   include "initContinuityErrs.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    for (runTime++; !runTime.end(); runTime++)
    {
        Info<< "Time = " << runTime.timeName() << nl << endl;



#       include "readPISOControls.H"
#       include "CourantNo.H"

//////////////////////////////////////////////////////////////////////////////

volTensorField Gr=fvc::grad(U);
volTensorField E = (symm(Gr))&I;
gammadot = sqrt( (2.0*(E && E)) ); // shear rate


nu1=nu0 * pow( (1.0- (c/cm)), -2.0); // suspension viscosity
nu2=0.75*nu0*sqr(c/cm)*pow( (1.0- (c/cm)), -2.0);
fc=(1.0-(c/cm))*pow((1.0-c),alpha-1.0); // Sedimentation Hindrance function


Umx=max(max(U.component(0),U.component(1)),U.component(2)); // For correction purposes

SigmaL = -nu2 * ( gammadot+ (epsilon*Umx) ) * Q;
SigmaR = 2.0 * (nu1 - nu0)  * E;

SigmaF = (-p*I) + (2.0*nu0*E);

SigmaP = (SigmaL + SigmaR);

Sigma = SigmaP + SigmaF;
 
// Migration Flux Vector
J = ((2.0*sqr(a))/(9.0*nu0))*fc*fvc::div(SigmaP);
/////////////////////////////////////////////////////////////////////////////

//*********************************************************************************************//       




        // Pressure-velocity SIMPLE corrector
        for (int corr=0; corr<nCorr; corr++)
        {



            // Momentum predictor

            tmp<fvVectorMatrix> UEqn
            (
    
               - fvm::laplacian(nu1, U) -fvc::div(nu1*dev(fvc::grad(U)().T())) -fvc::div(SigmaL)
            );

            UEqn().relax();

            solve(UEqn() == -fvc::grad(p));

            p.boundaryField().updateCoeffs();
            volScalarField rUA = 1.0/UEqn().A();
            U = rUA*UEqn().H();
            UEqn.clear();
            phi = fvc::interpolate(U) & mesh.Sf();
            adjustPhi(phi, U, p);

            // Store pressure for under-relaxation
            p.storePrevIter();

            // Non-orthogonal pressure corrector loop
            for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
            {
                fvScalarMatrix pEqn
                (
                    fvm::laplacian(rUA, p) == fvc::div(phi)
                );

                pEqn.setReference(pRefCell, pRefValue);
                pEqn.solve();

                if (nonOrth == nNonOrthCorr)
                {
                    phi -= pEqn.flux();
                }
            }

#           include "continuityErrs.H"

            // Explicitly relax pressure for momentum corrector
            p.relax();

            // Momentum corrector
            U -= rUA*fvc::grad(p);
            U.correctBoundaryConditions();
        }



///////////////  concentration Evolution Equation ///////////////////////////

	fvScalarMatrix cEqn
        (
            
          fvm::ddt(c) + fvm::div(phi,c)
       
        );
	    

		solve (  cEqn == -fvc::div(J)  );




	    	  
///////////////////////////////////////////////////////////////////////////////////

        runTime.write();

        Info<< "ExecutionTime = "
            << runTime.elapsedCpuTime()
            << " s\n\n" << endl;
    }

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
